import java.util.Random;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class CreaturePanel extends JPanel
{
	// Constants
	public final static int PANEL_WIDTH = 500;
	public final static int PANEL_HEIGHT = 500;
	public final static int TIME_BASE_MS = 600;
	public final static int TIME_VAR_MS = 200;

	
	// Static variables
	private static Random rand = new Random();
	
	// Instance variables
	private Creature creature;
	private JLabel countLabel;
	private Timer timer;
	private int count;
	
	// Constructor
	public CreaturePanel(Creature creature)
	{
		// Add creature
		this.creature = creature;
		// Set count and countLabel 
		count = 0;
		countLabel = new JLabel("Number of times caught: " + count);
		add(countLabel);
		// Add mouse listener 
		addMouseListener(new CreatureClickListener());
		// Set preferred size
		setPreferredSize(new Dimension(PANEL_WIDTH, PANEL_HEIGHT));	
		// Create timer and reset
		int timeDelay = TIME_BASE_MS - TIME_VAR_MS + 2*rand.nextInt(TIME_VAR_MS+1);
		timer = new Timer(timeDelay , new TimerListener());
		timer.start();
		// Randomize creature location
		randomizeCreatureLocation();
	}
	
	// Paint component
	public void paintComponent(Graphics g)
	{
		// Call super method
		super.paintComponent(g);
		// Set backgroung color
		setBackground(Color.white);
		// Draw the creature at the location given by the creature object
		g.drawImage(creature.getImage(), creature.getX_loc(), creature.getY_loc(), null);
	}
	
	// Method for placing creature randomly, also calls repaint
	public void randomizeCreatureLocation()
	{
		// Choose random location for the creature
		int newXLoc = rand.nextInt(PANEL_WIDTH-creature.getWidth());
		int newYLoc = rand.nextInt(PANEL_HEIGHT-creature.getHeight());
		creature.setLocation(newXLoc, newYLoc);
		repaint();
	}
	
	// Private class for listening to mouse events
	private class CreatureClickListener implements MouseListener
	{
		public void mousePressed(MouseEvent e)
		{
			// Check if the user clicked on the creature
			boolean success = creature.clickInImage(e.getX(), e.getY());
			
			// If successful, add one to the point total
			if (success)
			{           
				// Add one to count and display 
				count++;
				countLabel.setText("Number of times caught: " + count);
			}
		}
		 
		// Unused events
		public void	mouseClicked(MouseEvent e) {}
      public void mouseEntered(MouseEvent e){} 
      public void mouseExited(MouseEvent e){}
      public void mouseReleased(MouseEvent e){} 
	}
	
	// Private class for listening to timer events
	private class TimerListener implements ActionListener
	{
		// Called after every given amount of time
		public void actionPerformed(ActionEvent e)
		{
			// Changes the creature's location
			randomizeCreatureLocation();
			int timeDelay = TIME_BASE_MS - TIME_VAR_MS + 2*rand.nextInt(TIME_VAR_MS+1);
			timer.setDelay(timeDelay);
		}
	}
}